==> eopf-2.3.0-py3-none-any.whl: eopf/computing/utils.py <==
from typing import Any, Dict, Optional, Tuple

import dask.array as da
import numpy as np
from datatree import DataTree

from eopf import EOContainer
from eopf.computing import eopf_breakpoint_decorator
from eopf.computing.abstract import ADF, EOProcessingUnit
from eopf.product import EOProduct
from eopf.product.eo_group import EOGroup
from eopf.product.eo_variable import EOVariable


class EOExtractRoiUnit(EOProcessingUnit):
    """Processing unit responsible of extracting of the region of interest to be processed"""

    @eopf_breakpoint_decorator(identifier="EOExtractRoiUnit")
    def run(
        self,
        inputs: dict[str, EOProduct | EOContainer | DataTree[Any]],
        adfs: Optional[dict[str, ADF]] = None,
        **kwargs: Any,
    ) -> dict[str, EOProduct | EOContainer | DataTree[Any]]:
        """
        Processing unit implementation method for ROI filtering.
        EOContainer are not allowed
        Parameters
        ----------
        inputs: dict[str, EOProduct| EOContainer| DataTree]
            The products from which we want to extract the region of interest.

        adfs: dict[str,ADF]
            all the ADFs needed to process

        region: Str
            region of interest to be extracted as string "(x,y,w,h)", w,h positive
            x and y can be indices or the geodetic coordinates lon and lat depending on the region type
        type: Str
            the region type, can be one of the following :
            "slice" or "geo_region"

        ref_variable: Str, optional
            processing context, not used by this processing unit


        Returns
        -------
        dict[str, EOProduct|EOContainer|DataTree]
            containing the ROI processed
        """
        region: str = kwargs["region"]
        ref_variable: Optional[str] = kwargs.get("ref_variable", None)
        import ast

        # return the ROI
        ret: dict[str, EOProduct | EOContainer | DataTree[Any]] = {}
        for key, eoproduct in inputs.items():
            if isinstance(eoproduct, (EOContainer, DataTree)):
                raise TypeError("EOContainer not allowed on this processing Unit")
            ret[key] = eoproduct.subset(region=ast.literal_eval(region), reference=ref_variable)
        return ret


class EORechunkingUnit(EOProcessingUnit):
    """Processing unit responsible of re-chunking all EOProduct variables"""

    def run(
        self,
        inputs: dict[str, EOProduct | EOContainer | DataTree[Any]],
        adfs: Optional[dict[str, ADF]] = None,
        **kwargs: Any,
    ) -> dict[str, EOProduct | EOContainer | DataTree[Any]]:
        """
        Processing unit implementation method for the rechunking.
        Works inplace and modifies the input eoproducts

        Parameters
        ----------
        inputs: dict[str, EOProduct | EOContainer | DataTree]
            The product that is going to be re-chunked.

        adfs: dict[str,ADF]
            all the ADFs needed to process

        chunks: Dict
            new chunks to be applied : {'rows':1000,'columns' : -1}
            -1 indicates the full size of the corresponding dimension.

        Returns
        -------
        dict[str, EOProduct|DataTree]
            rechunked inputs using the provided dimensions sizes
        """
        chunks: Dict[str, int] = kwargs["chunks"]
        for key, eoproduct in inputs.items():
            if not isinstance(eoproduct, DataTree):
                self.rechunk_product(eoproduct, chunks)
        return inputs

    def rechunk_product(self, prod: EOProduct | EOContainer, chunks: Dict[str, int]) -> None:
        if isinstance(prod, EOContainer):
            for prod_keys in prod.keys():
                self.rechunk_product(prod[prod_keys], chunks)
        elif isinstance(prod, EOProduct):
            self.rechunk_all_variables(prod, chunks)

    def rechunk_all_variables(
        self,
        eogroup: EOGroup,
        chunks: Dict[str, int],
    ) -> None:
        """
        re-chunk all variables of a given EOProduct or EOGroup using the new dimensions provided within chunks dict.
        In-place modifier
        """
        # loop over all groups of the eocontainer
        for _, group in eogroup.groups:
            # do the same thing for all sub-groups
            self.rechunk_all_variables(group, chunks)
            # re-chunk all concerned variables
            for _, var in group.variables:
                new_chunks = {}
                for dim, chunk in chunks.items():
                    if dim in var.dims:
                        new_chunks[dim] = chunk
                # re-chunk the variable
                if len(new_chunks) > 0:
                    var.chunk(new_chunks)


class EOConstantUnit(EOProcessingUnit):
    """Add a constant variable in the product"""

    def run(
        self,
        inputs: dict[str, EOProduct | EOContainer | DataTree[Any]],
        adfs: Optional[dict[str, ADF]] = None,
        **kwargs: Any,
    ) -> dict[str, EOProduct | EOContainer | DataTree[Any]]:
        """
        Add a new variable with the given name to an EOProduct
        EOContainer are not allowed
        Parameters
        ----------
        inputs: dict[str, EOProduct | EOContainer | DataTree[Any]]
            The product that is going to be modified.

        adfs: dict[str,ADF]
            all the ADFs needed to process

        value: np.float64
            constant value of the new variable to add.

        variable_path: str
            path to the variable in the eoproduct

        variable_name: str
            name of he variable

        shape: Tuple[int,...]
            shape of the new variable

        Returns
        -------
        dict[str, EOProduct| EOContainer | DataTree]
            input with the new variable added
        """
        value: np.float64 = kwargs["value"]
        variable_path: str = kwargs["variable_path"]
        shape: Tuple[int, ...] = kwargs["shape"]
        for key, eoproduct in inputs.items():
            if isinstance(eoproduct, (EOContainer, DataTree)):
                raise TypeError("EOContainer not allowed on this ProcessingUnit")
            eoproduct[variable_path] = EOVariable(data=da.full(shape, value, dtype=np.float64))
        return inputs


class EODummyUnit(EOProcessingUnit):
    """Dummy unit that returns the inputs"""

    @eopf_breakpoint_decorator(
        identifier="dummy",
        description="Dummy breakpoint for tutorial purpose",
        filename_prefix="Dummy_",
    )
    def run(
        self,
        inputs: dict[str, EOProduct | DataTree[Any]],
        adfs: Optional[dict[str, ADF]] = None,
        **kwargs: Any,
    ) -> dict[str, EOProduct | DataTree[Any]]:
        """
        Return the input
        ----------
        inputs: dict[str, EOProduct|DataTree]
            The product that is going to be modified.

        adfs: dict[str,ADF]
            all the ADFs needed to process

        Returns
        -------
        dict[str, EOProduct|DataTree]
            inputs
        """
        return inputs

==> eopf-2.5.5-py3-none-any.whl: eopf/computing/utils.py <==
from typing import Any, Dict, List, Optional, Tuple

import dask.array as da
import numpy as np
from xarray.core.datatree import DataTree

from eopf import EOContainer
from eopf.computing import eopf_breakpoint_decorator
from eopf.computing.abstract import (
    DataType,
    EOProcessingUnit,
    MappingAuxiliary,
    MappingDataType,
)
from eopf.product import EOProduct
from eopf.product.eo_group import EOGroup
from eopf.product.eo_variable import EOVariable


class EOExtractRoiUnit(EOProcessingUnit):
    """Processing unit responsible of extracting of the region of interest to be processed"""

    @staticmethod
    def get_available_modes() -> List[str]:
        return ["nominal", "passthrought"]

    @staticmethod
    def get_default_mode() -> str:
        return "nominal"

    @eopf_breakpoint_decorator(identifier="EOExtractRoiUnit")
    def run(
        self,
        inputs: MappingDataType,
        adfs: Optional[MappingAuxiliary] = None,
        **kwargs: Any,
    ) -> MappingDataType:  # pragma: no cover
        """
        Processing unit implementation method for ROI filtering.
        EOContainer are not allowed
        Parameters
        ----------
        inputs: Mapping[str, DataType]
            The products from which we want to extract the region of interest.

        adfs: Mapping[str,AuxiliaryDataFile]
            all the ADFs needed to process

        region: Str
            region of interest to be extracted as string "(x,y,w,h)", w,h positive
            x and y can be indices or the geodetic coordinates lon and lat depending on the region type
        type: Str
            the region type, can be one of the following :
            "slice" or "geo_region"

        ref_variable: Str, optional
            processing context, not used by this processing unit


        Returns
        -------
        Mapping[str, DataType]
            containing the ROI processed
        """
        region: str = kwargs["region"]
        ref_variable: Optional[str] = kwargs.get("ref_variable", None)
        import ast

        # return the ROI
        ret: dict[str, DataType] = {}
        for key, eoproduct in inputs.items():
            if isinstance(eoproduct, (EOContainer, DataTree)):
                raise TypeError("EOContainer not allowed on this processing Unit")
            ret[key] = eoproduct.subset(region=ast.literal_eval(region), reference=ref_variable)
        return ret


class EORechunkingUnit(EOProcessingUnit):
    """Processing unit responsible of re-chunking all EOProduct variables"""

    def run(
        self,
        inputs: MappingDataType,
        adfs: Optional[MappingAuxiliary] = None,
        **kwargs: Any,
    ) -> MappingDataType:  # pragma: no cover
        """
        Processing unit implementation method for the rechunking.
        Works inplace and modifies the input eoproducts

        Parameters
        ----------
        inputs: Mapping[str, EOProduct | EOContainer | DataTree]
            The product that is going to be re-chunked.

        adfs: Mapping[str,AuxiliaryDataFile]
            all the ADFs needed to process

        chunks: Dict
            new chunks to be applied : {'rows':1000,'columns' : -1}
            -1 indicates the full size of the corresponding dimension.

        Returns
        -------
        Mapping[str, EOProduct|DataTree]
            rechunked inputs using the provided dimensions sizes
        """
        chunks: Dict[str, int] = kwargs["chunks"]
        for key, eoproduct in inputs.items():
            if not isinstance(eoproduct, DataTree):
                self.rechunk_product(eoproduct, chunks)
        return inputs

    def rechunk_product(self, prod: DataType, chunks: Dict[str, int]) -> None:
        if isinstance(prod, EOContainer):
            for prod_keys in prod.keys():
                self.rechunk_product(prod[prod_keys], chunks)
        elif isinstance(prod, EOProduct):
            self.rechunk_all_variables(prod, chunks)

    def rechunk_all_variables(
        self,
        eogroup: EOGroup,
        chunks: Dict[str, int],
    ) -> None:
        """
        re-chunk all variables of a given EOProduct or EOGroup using the new dimensions provided within chunks dict.
        In-place modifier
        """
        # loop over all groups of the eocontainer
        for _, group in eogroup.groups:
            # do the same thing for all sub-groups
            self.rechunk_all_variables(group, chunks)
            # re-chunk all concerned variables
            for _, var in group.variables:
                new_chunks = {}
                for dim, chunk in chunks.items():
                    if dim in var.dims:
                        new_chunks[dim] = chunk
                # re-chunk the variable
                if len(new_chunks) > 0:
                    var.chunk(new_chunks)


class EOConstantUnit(EOProcessingUnit):
    """Add a constant variable in the product"""

    def run(
        self,
        inputs: MappingDataType,
        adfs: Optional[MappingAuxiliary] = None,
        **kwargs: Any,
    ) -> MappingDataType:  # pragma: no cover
        """
        Add a new variable with the given name to an EOProduct
        EOContainer are not allowed
        Parameters
        ----------
        inputs: Mapping[str, DataType]
            The product that is going to be modified.

        adfs: Mapping[str,AuxiliaryDataFile]
            all the ADFs needed to process

        value: np.float64
            constant value of the new variable to add.

        variable_path: str
            path to the variable in the eoproduct

        variable_name: str
            name of he variable

        shape: Tuple[int,...]
            shape of the new variable

        Returns
        -------
        Mapping[str, EOProduct| EOContainer | DataTree]
            input with the new variable added
        """
        value: np.float64 = kwargs["value"]
        variable_path: str = kwargs["variable_path"]
        shape: Tuple[int, ...] = kwargs["shape"]
        for key, eoproduct in inputs.items():
            if isinstance(eoproduct, (EOContainer, DataTree)):
                raise TypeError("EOContainer not allowed on this ProcessingUnit")
            eoproduct[variable_path] = EOVariable(data=da.full(shape, value, dtype=np.float64))
        return inputs


class EODummyUnit(EOProcessingUnit):
    """Dummy unit that returns the inputs"""

    @eopf_breakpoint_decorator(
        identifier="dummy",
        description="Dummy breakpoint for tutorial purpose",
        filename_prefix="Dummy_",
    )
    def run(
        self,
        inputs: MappingDataType,
        adfs: Optional[MappingAuxiliary] = None,
        **kwargs: Any,
    ) -> MappingDataType:  # pragma: no cover
        """
        Return the input
        ----------
        inputs: Mapping[str, DataType]
            The product that is going to be modified.

        adfs: Mapping[str,ADF]
            all the ADFs needed to process

        Returns
        -------
        Mapping[str, DataType]
            inputs
        """
        return inputs
